#define _POSIX_C_SOURCE 200809L
#include <unistd.h>           
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdint.h>

#include "file_blk_dev.h"

/**
 * Vytvori blokove zarizeni simulovane souborem. Zarizeni ma minimalne velikost dev_size bytu
 * a pracuje s bloky o velikosti block_size bytu
 */
int file_blk_dev_create(struct file_blk_dev *dev, char *file_name, uint32_t block_size, uint32_t dev_size)
{
	int fd = open(file_name, O_CREAT | O_RDWR, 0644);
	if (fd < 0) return EDEV_ERR;

	/* nastavi velikost souboru */
	uint32_t block_cnt = dev_size / block_size;
	if (ftruncate(fd, block_size * block_cnt) < 0) return EDEV_ERR;

	dev->fd = fd;
	dev->blk_dev.block_size = block_size;
	dev->blk_dev.block_cnt  = block_cnt; 
	dev->blk_dev.read       = file_blk_dev_read;
	dev->blk_dev.write      = file_blk_dev_write;
	return EOK;
}

int file_blk_dev_read(struct blk_dev *dev, unsigned char *buf, uint32_t blk_offset, uint32_t blk_cnt)
{
	struct file_blk_dev *fblk_dev = (struct file_blk_dev *) dev;
	uint32_t data_size = dev->block_size * blk_cnt;

	if (lseek(fblk_dev->fd, blk_offset * dev->block_size, SEEK_SET) < 0) return EDEV_ERR;
	if (read(fblk_dev->fd, buf, data_size) != data_size) return EDEV_ERR;

	return EOK;
}

int file_blk_dev_write(struct blk_dev *dev, unsigned char *buf, uint32_t blk_offset, uint32_t blk_cnt)
{
	struct file_blk_dev *fblk_dev = (struct file_blk_dev *) dev;
	uint32_t data_size = dev->block_size * blk_cnt;

	if (lseek(fblk_dev->fd, blk_offset * dev->block_size, SEEK_SET) < 0) return EDEV_ERR;
	if (write(fblk_dev->fd, buf, data_size) != data_size) return EDEV_ERR;
	return EOK;
}

int file_blk_dev_close(struct file_blk_dev *dev)
{
	if (close(dev->fd) == 0) return EOK;
	return EDEV_ERR;
}

